-- ----------------------------------------------------------------
-- SASAPP Schema definition script: Workflow
-- Copyright (c) 2010-2011 by SAS Institute Inc., Cary, NC 27513
-- 
-- Supported DBMS: MSSQL
-- ----------------------------------------------------------------



create table SAS_WF_CONDITIONAL_CHANGE
(
   REAL_ID bigint not null,
   PRECONDITION_FLG tinyint null,
   CONDITION_TYPE_NO int null,
   CHANGE_TYPE_NO int null,
   PROCESS_ID bigint null,
   PARENT_ID bigint null,
   CONDITION_TARGET_PROCESS_ID bigint null,
   CONDITION_TARGET_STATE_ID int null,
   CONDITION_TARGET_STATUS_ID bigint null,
   CHANGE_STATUS_ID bigint null,
   CHANGE_STATE_NO int null,
   primary key (REAL_ID)
);

create table SAS_WF_EVENT
(
   REAL_ID bigint not null,
   NAME_NM varchar(255) null,
   SOURCE_TXT varchar(255) null,
   SCOPE_ID varchar(255) null,
   primary key (REAL_ID)
);

create table SAS_WF_MEDIATION_OBJECT
(
   REAL_ID bigint not null,
   WORKFLOW_OBJECT_ID varchar(255) not null unique,
   LABEL_NM varchar(255) null,
   primary key (REAL_ID)
);

create table SAS_WF_OPERAND
(
   REAL_ID bigint not null,
   PROCESS_ID bigint null,
   IS_ROOT_FLG tinyint null,
   DESCRIPTION_TXT varchar(4000) null,
   SCOPE_FLG tinyint null,
   INDEX_VALUE_TXT varchar(4000) null,
   LARGE_VALUE_TXT varbinary(MAX) null,
   XML_VALUE_TXT varbinary(MAX) null,
   SCHEMA_TXT varbinary(MAX) null,
   WORKFLOW_OBJECT_ID varchar(255) not null unique,
   LABEL_NM varchar(255) null,
   NUMBER_VALUE_NO numeric(19,2) null,
   DATE_VALUE_DTTM datetime null,
   LABELED_ENTITY_ID  varchar(255),
   RB_KEY_FOR_NAME_TXT varchar(255),
   RB_KEY_FOR_DESC_TXT varchar(255),   
   primary key (REAL_ID)
);


create table SAS_WF_OPERAND_ATTRIBUTES
(
   OPERAND_ID bigint not null,
   VALUE_TXT varchar(4000) null,
   NAME_NM varchar(255) not null,
   primary key (OPERAND_ID, NAME_NM)
);

create table SAS_WF_OPERAND_METADATA
(
   OPERAND_ID bigint not null,
   VALUE_TXT varbinary(MAX) null,
   NAME_NM varchar(255) not null,
   primary key (OPERAND_ID, NAME_NM)
);

create table SAS_WF_PERSISTENT
(
   REAL_ID bigint not null,
   primary key (REAL_ID)
);

create table SAS_WF_POLICY
(
   REAL_ID bigint not null,
   PROCESS_ID bigint null,
   IS_ROOT_FLG tinyint null,
   DESCRIPTION_TXT varchar(4000) null,
   EVENT_SOURCE_TXT varchar(255) null,
   EVENT_NAME_TXT varchar(255) null,
   LANGUAGE_TXT varchar(255) null,
   SECURITY_POLICY_TXT varchar(4000) null,
   SOURCE_CODE_TXT varchar(4000) null,
   SOURCE_CODE_TYPE_TXT int null,
   WORKFLOW_OBJECT_ID varchar(255) not null unique,
   LABEL_NM varchar(255) null,
   LABELED_ENTITY_ID  varchar(255),
   RB_KEY_FOR_NAME_TXT varchar(255),
   RB_KEY_FOR_DESC_TXT varchar(255),      
   primary key (REAL_ID)
);



create table SAS_WF_POLICY_ATTRIBUTES
(
   POLICY_ID bigint not null,
   VALUE_TXT varchar(4000) null,
   NAME_NM varchar(255) not null,
   primary key (POLICY_ID, NAME_NM)
);

create table SAS_WF_POLICY_LP_EVENT_ATTR
(
   POLICY_ID bigint not null,
   VALUE_TXT varchar(4000) null,
   NAME_NM varchar(255) not null,
   primary key (POLICY_ID, NAME_NM)
);

create table SAS_WF_POLICY_METADATA
(
   POLICY_ID bigint not null,
   VALUE_TXT varbinary(MAX) null,
   NAME_NM varchar(255) not null,
   primary key (POLICY_ID, NAME_NM)
);

create table SAS_WF_POLICY_STR_EVENT_ATTR
(
   POLICY_ID bigint not null,
   VALUE_TXT varchar(4000) null,
   NAME_NM varchar(255) not null,
   primary key (POLICY_ID, NAME_NM)
);

create table SAS_WF_PROCESS
(
   REAL_ID bigint not null,
   IS_DELETED_FLG tinyint null,
   STATE_NO int null,
   PARENT_ID bigint null,
   PROCESS_DEFINITION_ID bigint null,
   ROOT_PROCESS_ID bigint null,
   IS_ROOT_FLG tinyint null,
   DESCRIPTION_TXT varchar(4000) null,
   IS_DEFINITION_FLG tinyint null,
   START_DATE_DTTM datetime null,
   END_DATE_DTTM datetime null,
   ACCESS_TAG_ID bigint null,
   SCHEDULED_DATE_DTTM datetime null,
   DUE_DATE_DTTM datetime null,
   IS_LOCKED_FLG tinyint null,
   WORKFLOW_OBJECT_ID varchar(255) not null unique,
   LABEL_NM varchar(255) null,
   VERSION_NM varchar(255) null,
   LABELED_ENTITY_ID  varchar(255),
   RB_KEY_FOR_NAME_TXT varchar(255),
   RB_KEY_FOR_DESC_TXT varchar(255),   
   primary key (REAL_ID)
);

create table SAS_WF_PROCESS_ACE
(
   REAL_ID bigint not null,
   ASSIGNEE_ID bigint null,
   TYPE_NO int null,
   NAME_NM varchar(255) null,
   ROLE_NM varchar(255) null,
   primary key (REAL_ID)
);

create table SAS_WF_PROCESS_ATTR
(
   PROCESS_ID bigint not null,
   VALUE_TXT varchar(4000) null,
   NAME_NM varchar(255) not null,
   primary key (PROCESS_ID, NAME_NM)
);

create table SAS_WF_PROCESS_CURRENT_STATUS
(
   PROCESS_ID bigint not null,
   STATUS_ID bigint not null,
   primary key (PROCESS_ID, STATUS_ID)
);

create table SAS_WF_PROCESS_METADATA
(
   PROCESS_ID bigint not null,
   VALUE_TXT varbinary(MAX) null,
   NAME_NM varchar(255) not null,
   primary key (PROCESS_ID, NAME_NM)
);

create table SAS_WF_STATUS
(
   REAL_ID bigint not null,
   PROCESS_ID bigint null,
   IS_ROOT_FLG tinyint null,
   TYPE_TXT varchar(4000) null,
   SCOPE_FLG tinyint null,
   WORKFLOW_OBJECT_ID varchar(255) not null unique,
   LABEL_NM varchar(255) null,
   LABELED_ENTITY_ID  varchar(255),
   RB_KEY_FOR_NAME_TXT varchar(255),
   RB_KEY_FOR_DESC_TXT varchar(255),   
   primary key (REAL_ID)
);



create table SAS_WF_STATUS_ATTRIBUTES
(
   STATUS_ID bigint not null,
   VALUE_TXT varchar(4000) null,
   NAME_NM varchar(255) not null,
   primary key (STATUS_ID, NAME_NM)
);

create table SAS_WF_STATUS_METADATA
(
   STATUS_ID bigint not null,
   VALUE_TXT varbinary(MAX) null,
   NAME_NM varchar(255) not null,
   primary key (STATUS_ID, NAME_NM)
);

create table SAS_WF_TIMERSPEC
(
   REAL_ID bigint not null,
   PROCESS_ID bigint null,
   SCHEDULE_DATE_DTTM datetime null,
   SCHEDULE_EXPRESSION_TXT varchar(255) null,
   REPEAT_EXPRESSION_TXT varchar(255) null,
   REPEAT_CNT int null,
   END_DATE_DTTM datetime null,
   END_EXPRESSION_TXT varchar(255) null,
   CALENDAR_TXT varchar(255) null,
   DESCRIPTION_TXT varchar(255) null,
   SAS_WORKFLOW_EVENT_NO bigint null,
   STARTED_TIMER_ID varchar(255) null,
   primary key (REAL_ID)
);

create table SAS_WF_TIMER_LP_EVENT_ATTR
(
   SAS_WORKFLOW_EVENT_ID bigint not null,
   VALUE_TXT varchar(4000) null,
   NAME_NM varchar(255) not null,
   primary key (SAS_WORKFLOW_EVENT_ID, NAME_NM)
);

create table SAS_WF_TIMER_STRING_EVENT_ATTR
(
   SAS_WORKLFOW_EVENT_ID bigint not null,
   VALUE_TXT varchar(4000) null,
   NAME_NM varchar(255) not null,
   primary key (SAS_WORKLFOW_EVENT_ID, NAME_NM)
);

create table SAS_WF_TAGS
(
    TAG_ID bigint not null,
    TAG_NM varchar(255) not null unique,
    primary key (TAG_ID)
);

create table SAS_WF_TAGS_ASSIGNED
(
    PROCESS_ID bigint not null,
    TAG_ID bigint not null
);

create index CONDCHANGE_TARGET_PROC_IX on SAS_WF_CONDITIONAL_CHANGE (CONDITION_TARGET_PROCESS_ID);

create index CONDCHANGE_PROC_IX on SAS_WF_CONDITIONAL_CHANGE (PROCESS_ID);

create index CONDCHANGE_PARENT_IX on SAS_WF_CONDITIONAL_CHANGE (PARENT_ID);

alter table SAS_WF_CONDITIONAL_CHANGE
   add constraint CONDITIONAL_CHANGE_FK01
   foreign key (CHANGE_STATUS_ID) references SAS_WF_STATUS;

alter table SAS_WF_CONDITIONAL_CHANGE
   add constraint CONDITIONAL_CHANGE_FK02
   foreign key (CONDITION_TARGET_STATUS_ID) references SAS_WF_STATUS;

alter table SAS_WF_CONDITIONAL_CHANGE
   add constraint CONDITIONAL_CHANGE_FK03
   foreign key (CONDITION_TARGET_PROCESS_ID) references SAS_WF_PROCESS;

alter table SAS_WF_CONDITIONAL_CHANGE
   add constraint CONDITIONAL_CHANGE_FK04
   foreign key (PARENT_ID) references SAS_WF_CONDITIONAL_CHANGE;

alter table SAS_WF_CONDITIONAL_CHANGE
   add constraint CONDITIONAL_CHANGE_FK05
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;

create index OPERAND_ISROOT_IX on SAS_WF_OPERAND (IS_ROOT_FLG);

create index OPERAND_PROCESS_IX on SAS_WF_OPERAND (PROCESS_ID);

alter table SAS_WF_OPERAND
   add constraint OPERAND_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;


alter table SAS_WF_OPERAND_ATTRIBUTES
   add constraint OPERAND_ATTRIBUTES_FK01
   foreign key (OPERAND_ID) references SAS_WF_OPERAND;

alter table SAS_WF_OPERAND_METADATA
   add constraint OPERAND_METADATA_FK01
   foreign key (OPERAND_ID) references SAS_WF_OPERAND;

create index POLICY_ISROOT_IX on SAS_WF_POLICY (IS_ROOT_FLG);

create index POLICY_PROCESS_IX on SAS_WF_POLICY (PROCESS_ID);

alter table SAS_WF_POLICY
   add constraint POLICY_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;


alter table SAS_WF_POLICY_ATTRIBUTES
   add constraint POLICY_ATTRIBUTES_FK01
   foreign key (POLICY_ID) references SAS_WF_POLICY;

alter table SAS_WF_POLICY_LP_EVENT_ATTR
   add constraint POLICY_LP_EVENT_ATTR_FK01
   foreign key (POLICY_ID) references SAS_WF_POLICY;

alter table SAS_WF_POLICY_METADATA
   add constraint POLICY_METADATA_FK01
   foreign key (POLICY_ID) references SAS_WF_POLICY;

alter table SAS_WF_POLICY_STR_EVENT_ATTR
   add constraint POLICY_STR_EVENT_ATTR_FK01
   foreign key (POLICY_ID) references SAS_WF_POLICY;

create index PROCESS_ISROOT_IX on SAS_WF_PROCESS (IS_ROOT_FLG);

create index PROCESS_ROOT_IX on SAS_WF_PROCESS (ROOT_PROCESS_ID);

create index PROCESS_ISDEFINITION_IX on SAS_WF_PROCESS (IS_DEFINITION_FLG);

create index PROCESS_IS_LOCKED_IX on SAS_WF_PROCESS (IS_LOCKED_FLG);

create index PROCESS_DEFINITION_IX on SAS_WF_PROCESS (PROCESS_DEFINITION_ID);

create index PROCESS_PARENT_IX on SAS_WF_PROCESS (PARENT_ID);

create index PROCESS_STATE_IX on SAS_WF_PROCESS (STATE_NO);

alter table SAS_WF_PROCESS
   add constraint PROCESS_FK01
   foreign key (ROOT_PROCESS_ID) references SAS_WF_PROCESS;

alter table SAS_WF_PROCESS
   add constraint PROCESS_FK02
   foreign key (PROCESS_DEFINITION_ID) references SAS_WF_PROCESS;

alter table SAS_WF_PROCESS
   add constraint PROCESS_FK03
   foreign key (PARENT_ID) references SAS_WF_PROCESS;

create index PROCESS_ACE_TYPE_IX on SAS_WF_PROCESS_ACE (TYPE_NO);

create index PROCESSACE_ASSIGNEE_IX on SAS_WF_PROCESS_ACE (ASSIGNEE_ID);

alter table SAS_WF_PROCESS_ACE
   add constraint PROCESS_ACE_FK01
   foreign key (ASSIGNEE_ID) references SAS_WF_PROCESS;

alter table SAS_WF_PROCESS_ATTR
   add constraint PROCESS_ATTR_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;

alter table SAS_WF_PROCESS_CURRENT_STATUS
   add constraint PROCESS_CURRENT_STATUS_FK02
   foreign key (STATUS_ID) references SAS_WF_STATUS;

alter table SAS_WF_PROCESS_CURRENT_STATUS
   add constraint PROCESS_CURRENT_STATUS_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;

alter table SAS_WF_PROCESS_METADATA
   add constraint PROCESS_METADATA_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;

create index STATUS_ISROOT_IX on SAS_WF_STATUS (IS_ROOT_FLG);

create index STATUS_PROCESS_IX on SAS_WF_STATUS (PROCESS_ID);

alter table SAS_WF_STATUS
   add constraint STATUS_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;


alter table SAS_WF_STATUS_ATTRIBUTES
   add constraint STATUS_ATTRIBUTES_FK01
   foreign key (STATUS_ID) references SAS_WF_STATUS;

alter table SAS_WF_STATUS_METADATA
   add constraint STATUS_METADATA_FK01
   foreign key (STATUS_ID) references SAS_WF_STATUS;

create index TIMERSPEC_PROCESS_IX on SAS_WF_TIMERSPEC (PROCESS_ID);

alter table SAS_WF_TIMERSPEC
   add constraint TIMERSPEC_FK01
   foreign key (SAS_WORKFLOW_EVENT_NO) references SAS_WF_EVENT;

alter table SAS_WF_TIMERSPEC
   add constraint TIMERSPEC_FK02
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;

alter table SAS_WF_TIMER_LP_EVENT_ATTR
   add constraint TIMER_LP_EVENT_ATTR_FK01
   foreign key (SAS_WORKFLOW_EVENT_ID) references SAS_WF_EVENT;

alter table SAS_WF_TIMER_STRING_EVENT_ATTR
   add constraint TIMER_STRING_EVENT_ATTR_FK01
   foreign key (SAS_WORKLFOW_EVENT_ID) references SAS_WF_EVENT;
   

create index CONDCHANGE_CHG_STATUS_IX on SAS_WF_CONDITIONAL_CHANGE (CHANGE_STATUS_ID);

create index CONDCHANGE_TARGET_STATUS_IX on SAS_WF_CONDITIONAL_CHANGE (CONDITION_TARGET_STATUS_ID);

create index PROCESS_CS_STATUS_IX on SAS_WF_PROCESS_CURRENT_STATUS (STATUS_ID);

create index TIMERSPEC_WF_EVENT_NO_IX on SAS_WF_TIMERSPEC (SAS_WORKFLOW_EVENT_NO);

alter table SAS_WF_TAGS_ASSIGNED
   add constraint TAG_ID_FK01
   foreign key (TAG_ID) references SAS_WF_TAGS;

create index TAG_ID_IX on SAS_WF_TAGS_ASSIGNED (TAG_ID);
