-- ----------------------------------------------------------------
-- SASAPP Schema definition script: Workflow
-- Copyright (c) 2010-2011 by SAS Institute Inc., Cary, NC 27513
-- 
-- Supported DBMS: POSTGRES
-- ----------------------------------------------------------------



create table SAS_WF_CONDITIONAL_CHANGE
(
   REAL_ID bigint not null,
   PRECONDITION_FLG boolean,
   CONDITION_TYPE_NO integer,
   CHANGE_TYPE_NO integer,
   PROCESS_ID bigint,
   PARENT_ID bigint,
   CONDITION_TARGET_PROCESS_ID bigint,
   CONDITION_TARGET_STATE_ID integer,
   CONDITION_TARGET_STATUS_ID bigint,
   CHANGE_STATUS_ID bigint,
   CHANGE_STATE_NO integer,
       constraint WF_CONDITIONAL_CHANGE_PK primary key (REAL_ID)
);

create table SAS_WF_EVENT
(
   REAL_ID bigint not null,
   NAME_NM varchar(255),
   SOURCE_TXT varchar(255),
   SCOPE_ID varchar(255),
       constraint WF_EVENT_PK primary key (REAL_ID)
);

create table SAS_WF_MEDIATION_OBJECT
(
   REAL_ID bigint not null,
   WORKFLOW_OBJECT_ID varchar(255) not null unique,
   LABEL_NM varchar(255),
       constraint WF_MEDIATION_OBJECT_PK primary key (REAL_ID)
);

create table SAS_WF_OPERAND
(
   REAL_ID bigint not null,
   PROCESS_ID bigint,
   IS_ROOT_FLG boolean,
   DESCRIPTION_TXT varchar(4000),
   SCOPE_FLG boolean,
   INDEX_VALUE_TXT varchar(32000),
   LARGE_VALUE_TXT BYTEA,
   XML_VALUE_TXT BYTEA,
   SCHEMA_TXT BYTEA,
   WORKFLOW_OBJECT_ID varchar(255) not null unique,
   LABEL_NM varchar(255),
   NUMBER_VALUE_NO numeric(18,2),
   DATE_VALUE_DTTM timestamp,   
   LABELED_ENTITY_ID  varchar(255),
   RB_KEY_FOR_NAME_TXT varchar(255),
   RB_KEY_FOR_DESC_TXT varchar(255),
       constraint WF_OPERAND_PK primary key (REAL_ID)
);


create table SAS_WF_OPERAND_ATTRIBUTES
(
   OPERAND_ID bigint not null,
   VALUE_TXT varchar(4000),
   NAME_NM varchar(255) not null,
       constraint WF_OPERAND_ATTRIBUTES_PK primary key (OPERAND_ID, NAME_NM)
);

create table SAS_WF_OPERAND_METADATA
(
   OPERAND_ID bigint not null,
   VALUE_TXT  BYTEA,
   NAME_NM varchar(255) not null,
       constraint WF_OPERAND_METADATA_PK primary key (OPERAND_ID, NAME_NM)
);

create table SAS_WF_PERSISTENT
(
   REAL_ID bigint not null,
       constraint WF_PERSISTENT_PK primary key (REAL_ID)
);

create table SAS_WF_POLICY
(
   REAL_ID bigint not null,
   PROCESS_ID bigint,
   IS_ROOT_FLG boolean,
   DESCRIPTION_TXT varchar(4000),
   EVENT_SOURCE_TXT varchar(255),
   EVENT_NAME_TXT varchar(255),
   LANGUAGE_TXT varchar(255),
   SECURITY_POLICY_TXT varchar(4000),
   SOURCE_CODE_TXT varchar(4000),
   SOURCE_CODE_TYPE_TXT integer,
   WORKFLOW_OBJECT_ID varchar(255) not null unique,
   LABEL_NM varchar(255),
   LABELED_ENTITY_ID  varchar(255),
   RB_KEY_FOR_NAME_TXT varchar(255),
   RB_KEY_FOR_DESC_TXT varchar(255),
       constraint WF_POLICY_PK primary key (REAL_ID)
);


create table SAS_WF_POLICY_ATTRIBUTES
(
   POLICY_ID bigint not null,
   VALUE_TXT varchar(4000),
   NAME_NM varchar(255) not null,
       constraint WF_POLICY_ATTRIBUTES_PK primary key (POLICY_ID, NAME_NM)
);

create table SAS_WF_POLICY_LP_EVENT_ATTR
(
   POLICY_ID bigint not null,
   VALUE_TXT varchar(4000),
   NAME_NM varchar(255) not null,
       constraint WF_POLICY_LP_EVENT_ATTR_PK primary key (POLICY_ID, NAME_NM)
);

create table SAS_WF_POLICY_METADATA
(
   POLICY_ID bigint not null,
   VALUE_TXT  BYTEA,
   NAME_NM varchar(255) not null,
       constraint WF_POLICY_METADATA_PK primary key (POLICY_ID, NAME_NM)
);

create table SAS_WF_POLICY_STR_EVENT_ATTR
(
   POLICY_ID bigint not null,
   VALUE_TXT varchar(4000),
   NAME_NM varchar(255) not null,
       constraint WF_POLICY_STR_EVENT_ATTR_PK primary key (POLICY_ID, NAME_NM)
);

create table SAS_WF_PROCESS
(
   REAL_ID bigint not null,
   IS_DELETED_FLG boolean,
   STATE_NO integer,
   PARENT_ID bigint,
   PROCESS_DEFINITION_ID bigint,
   ROOT_PROCESS_ID bigint,
   IS_ROOT_FLG boolean,
   DESCRIPTION_TXT varchar(4000),
   IS_DEFINITION_FLG boolean,
   START_DATE_DTTM timestamp,
   END_DATE_DTTM timestamp,
   ACCESS_TAG_ID bigint,
   SCHEDULED_DATE_DTTM timestamp,
   DUE_DATE_DTTM timestamp,
   IS_LOCKED_FLG boolean,
   WORKFLOW_OBJECT_ID varchar(255) not null unique,
   LABEL_NM varchar(255),
   VERSION_NM varchar(255),
   LABELED_ENTITY_ID  varchar(255),
   RB_KEY_FOR_NAME_TXT varchar(255),
   RB_KEY_FOR_DESC_TXT varchar(255),
       constraint WF_PROCESS_PK primary key (REAL_ID)
);

create table SAS_WF_PROCESS_ACE
(
   REAL_ID bigint not null,
   ASSIGNEE_ID bigint,
   TYPE_NO integer,
   NAME_NM varchar(255),
   ROLE_NM varchar(255),
       constraint WF_PROCESS_ACE_PK primary key (REAL_ID)
);

create table SAS_WF_PROCESS_ATTR
(
   PROCESS_ID bigint not null,
   VALUE_TXT varchar(4000),
   NAME_NM varchar(255) not null,
       constraint WF_PROCESS_ATTR_PK primary key (PROCESS_ID, NAME_NM)
);

create table SAS_WF_PROCESS_CURRENT_STATUS
(
   PROCESS_ID bigint not null,
   STATUS_ID bigint not null,
       constraint WF_PROCESS_CURRENT_STATUS_PK primary key (PROCESS_ID, STATUS_ID)
);

create table SAS_WF_PROCESS_METADATA
(
   PROCESS_ID bigint not null,
   VALUE_TXT  BYTEA,
   NAME_NM varchar(255) not null,
       constraint WF_PROCESS_METADATA_PK primary key (PROCESS_ID, NAME_NM)
);

create table SAS_WF_STATUS
(
   REAL_ID bigint not null,
   PROCESS_ID bigint,
   IS_ROOT_FLG boolean,
   TYPE_TXT varchar(4000),
   SCOPE_FLG boolean,
   WORKFLOW_OBJECT_ID varchar(255) not null unique,
   LABEL_NM varchar(255),
   LABELED_ENTITY_ID  varchar(255),
   RB_KEY_FOR_NAME_TXT varchar(255),
   RB_KEY_FOR_DESC_TXT varchar(255),
       constraint WF_STATUS_PK primary key (REAL_ID)
);


create table SAS_WF_STATUS_ATTRIBUTES
(
   STATUS_ID bigint not null,
   VALUE_TXT varchar(4000),
   NAME_NM varchar(255) not null,
       constraint WF_STATUS_ATTRIBUTES_PK primary key (STATUS_ID, NAME_NM)
);

create table SAS_WF_STATUS_METADATA
(
   STATUS_ID bigint not null,
   VALUE_TXT  BYTEA,
   NAME_NM varchar(255) not null,
       constraint WF_STATUS_METADATA_PK primary key (STATUS_ID, NAME_NM)
);

create table SAS_WF_TIMERSPEC
(
   REAL_ID bigint not null,
   PROCESS_ID bigint,
   SCHEDULE_DATE_DTTM timestamp,
   SCHEDULE_EXPRESSION_TXT varchar(255),
   REPEAT_EXPRESSION_TXT varchar(255),
   REPEAT_CNT integer,
   END_DATE_DTTM timestamp,
   END_EXPRESSION_TXT varchar(255),
   CALENDAR_TXT varchar(255),
   DESCRIPTION_TXT varchar(255),
   SAS_WORKFLOW_EVENT_NO bigint,
   STARTED_TIMER_ID varchar(255),
       constraint WF_TIMERSPEC_PK primary key (REAL_ID)
);

create table SAS_WF_TIMER_LP_EVENT_ATTR
(
   SAS_WORKFLOW_EVENT_ID bigint not null,
   VALUE_TXT varchar(4000),
   NAME_NM varchar(255) not null,
       constraint WF_TIMER_LP_EVENT_ATTR_PK primary key (SAS_WORKFLOW_EVENT_ID, NAME_NM)
);

create table SAS_WF_TIMER_STRING_EVENT_ATTR
(
   SAS_WORKLFOW_EVENT_ID bigint not null,
   VALUE_TXT varchar(4000),
   NAME_NM varchar(255) not null,
       constraint WF_TIMER_STRING_EVENT_ATTR_PK primary key (SAS_WORKLFOW_EVENT_ID, NAME_NM)
);

create table SAS_WF_TAGS
(
    TAG_ID bigint not null,
    TAG_NM varchar(255) not null unique,
       constraint WF_TAGS_PK primary key (TAG_ID)
);

create table SAS_WF_TAGS_ASSIGNED
(
    PROCESS_ID bigint not null,
    TAG_ID bigint not null
);

create index CONDCHANGE_TARGET_PROC_IX on SAS_WF_CONDITIONAL_CHANGE (CONDITION_TARGET_PROCESS_ID);

create index CONDCHANGE_PROC_IX on SAS_WF_CONDITIONAL_CHANGE (PROCESS_ID);

create index CONDCHANGE_PARENT_IX on SAS_WF_CONDITIONAL_CHANGE (PARENT_ID);

alter table SAS_WF_CONDITIONAL_CHANGE
   add constraint CONDITIONAL_CHANGE_FK01
   foreign key (CHANGE_STATUS_ID) references SAS_WF_STATUS;

alter table SAS_WF_CONDITIONAL_CHANGE
   add constraint CONDITIONAL_CHANGE_FK02
   foreign key (CONDITION_TARGET_STATUS_ID) references SAS_WF_STATUS;

alter table SAS_WF_CONDITIONAL_CHANGE
   add constraint CONDITIONAL_CHANGE_FK03
   foreign key (CONDITION_TARGET_PROCESS_ID) references SAS_WF_PROCESS;

alter table SAS_WF_CONDITIONAL_CHANGE
   add constraint CONDITIONAL_CHANGE_FK04
   foreign key (PARENT_ID) references SAS_WF_CONDITIONAL_CHANGE;

alter table SAS_WF_CONDITIONAL_CHANGE
   add constraint CONDITIONAL_CHANGE_FK05
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;

create index OPERAND_ISROOT_IX on SAS_WF_OPERAND (IS_ROOT_FLG);

create index OPERAND_PROCESS_IX on SAS_WF_OPERAND (PROCESS_ID);

alter table SAS_WF_OPERAND
   add constraint OPERAND_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;


alter table SAS_WF_OPERAND_ATTRIBUTES
   add constraint OPERAND_ATTRIBUTES_FK01
   foreign key (OPERAND_ID) references SAS_WF_OPERAND;

alter table SAS_WF_OPERAND_METADATA
   add constraint OPERAND_METADATA_FK01
   foreign key (OPERAND_ID) references SAS_WF_OPERAND;

create index POLICY_ISROOT_IX on SAS_WF_POLICY (IS_ROOT_FLG);

create index POLICY_PROCESS_IX on SAS_WF_POLICY (PROCESS_ID);

alter table SAS_WF_POLICY
   add constraint POLICY_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;


alter table SAS_WF_POLICY_ATTRIBUTES
   add constraint POLICY_ATTRIBUTES_FK01
   foreign key (POLICY_ID) references SAS_WF_POLICY;

alter table SAS_WF_POLICY_LP_EVENT_ATTR
   add constraint POLICY_LP_EVENT_ATTR_FK01
   foreign key (POLICY_ID) references SAS_WF_POLICY;

alter table SAS_WF_POLICY_METADATA
   add constraint POLICY_METADATA_FK01
   foreign key (POLICY_ID) references SAS_WF_POLICY;

alter table SAS_WF_POLICY_STR_EVENT_ATTR
   add constraint POLICY_STR_EVENT_ATTR_FK01
   foreign key (POLICY_ID) references SAS_WF_POLICY;

create index PROCESS_ISROOT_IX on SAS_WF_PROCESS (IS_ROOT_FLG);

create index PROCESS_ROOT_IX on SAS_WF_PROCESS (ROOT_PROCESS_ID);

create index PROCESS_ISDEFINITION_IX on SAS_WF_PROCESS (IS_DEFINITION_FLG);

create index PROCESS_IS_LOCKED_IX on SAS_WF_PROCESS (IS_LOCKED_FLG);

create index PROCESS_DEFINITION_IX on SAS_WF_PROCESS (PROCESS_DEFINITION_ID);

create index PROCESS_PARENT_IX on SAS_WF_PROCESS (PARENT_ID);

create index PROCESS_STATE_IX on SAS_WF_PROCESS (STATE_NO);

alter table SAS_WF_PROCESS
   add constraint PROCESS_FK01
   foreign key (ROOT_PROCESS_ID) references SAS_WF_PROCESS;

alter table SAS_WF_PROCESS
   add constraint PROCESS_FK02
   foreign key (PROCESS_DEFINITION_ID) references SAS_WF_PROCESS;

alter table SAS_WF_PROCESS
   add constraint PROCESS_FK03
   foreign key (PARENT_ID) references SAS_WF_PROCESS;

create index PROCESS_ACE_TYPE_IX on SAS_WF_PROCESS_ACE (TYPE_NO);

create index PROCESSACE_ASSIGNEE_IX on SAS_WF_PROCESS_ACE (ASSIGNEE_ID);

alter table SAS_WF_PROCESS_ACE
   add constraint PROCESS_ACE_FK01
   foreign key (ASSIGNEE_ID) references SAS_WF_PROCESS;

alter table SAS_WF_PROCESS_ATTR
   add constraint PROCESS_ATTR_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;

alter table SAS_WF_PROCESS_CURRENT_STATUS
   add constraint PROCESS_CURRENT_STATUS_FK02
   foreign key (STATUS_ID) references SAS_WF_STATUS;

alter table SAS_WF_PROCESS_CURRENT_STATUS
   add constraint PROCESS_CURRENT_STATUS_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;

alter table SAS_WF_PROCESS_METADATA
   add constraint PROCESS_METADATA_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;

create index STATUS_ISROOT_IX on SAS_WF_STATUS (IS_ROOT_FLG);

create index STATUS_PROCESS_IX on SAS_WF_STATUS (PROCESS_ID);

alter table SAS_WF_STATUS
   add constraint STATUS_FK01
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;


alter table SAS_WF_STATUS_ATTRIBUTES
   add constraint STATUS_ATTRIBUTES_FK01
   foreign key (STATUS_ID) references SAS_WF_STATUS;

alter table SAS_WF_STATUS_METADATA
   add constraint STATUS_METADATA_FK01
   foreign key (STATUS_ID) references SAS_WF_STATUS;

create index TIMERSPEC_PROCESS_IX on SAS_WF_TIMERSPEC (PROCESS_ID);

alter table SAS_WF_TIMERSPEC
   add constraint TIMERSPEC_FK01
   foreign key (SAS_WORKFLOW_EVENT_NO) references SAS_WF_EVENT;

alter table SAS_WF_TIMERSPEC
   add constraint TIMERSPEC_FK02
   foreign key (PROCESS_ID) references SAS_WF_PROCESS;

alter table SAS_WF_TIMER_LP_EVENT_ATTR
   add constraint TIMER_LP_EVENT_ATTR_FK01
   foreign key (SAS_WORKFLOW_EVENT_ID) references SAS_WF_EVENT;

alter table SAS_WF_TIMER_STRING_EVENT_ATTR
   add constraint TIMER_STRING_EVENT_ATTR_FK01
   foreign key (SAS_WORKLFOW_EVENT_ID) references SAS_WF_EVENT;

alter table SAS_WF_TAGS_ASSIGNED
   add constraint TAG_ID_FK01
   foreign key (TAG_ID) references SAS_WF_TAGS;

create index TAG_ID_IX on SAS_WF_TAGS_ASSIGNED (TAG_ID);
